"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class BotFrameworkExtension {
    // return default Id
    constructor(_uid) {
        this._uid = _uid;
    }
    normalizeButton(button) {
        const value = button.url || button.value || `${this._uid}-${button.payload}` || button.title;
        switch (button.type) {
            case 'web_url': {
                return {
                    title: button.title,
                    type: 'openUrl',
                    value: value
                };
            }
            case 'postback': {
                return {
                    title: button.title,
                    type: 'postBack',
                    value: value
                };
            }
            default: {
                return button;
            }
        }
    }
    text(text, quickReplies) {
        if (quickReplies && quickReplies.length) {
            return {
                attachments: [
                    this.attachment({
                        text: text,
                        buttons: (quickReplies || []).map(button => {
                            return this.normalizeButton(button);
                        })
                    })
                ]
            };
        }
        return {
            text: text
        };
    }
    buttonTemplate(text, buttons) {
        return {
            attachments: [
                this.attachment({
                    text: text,
                    buttons: (buttons || []).map(button => {
                        return this.normalizeButton(button);
                    })
                })
            ]
        };
    }
    image(url, quickReplies) {
        return {
            attachments: [{
                    contentType: 'image/png',
                    contentUrl: url,
                }
            ]
        };
    }
    genericTemplate(items) {
        return {
            attachmentLayout: "carousel",
            attachments: items
        };
    }
    genericItem(title, subtitle, imageUrl, defaultActionUrl, buttons, assetId) {
        return this.attachment({
            title: title,
            subtitle: subtitle,
            images: [{
                    url: imageUrl
                }],
            buttons: (buttons || []).map(button => {
                switch (button.type) {
                    case 'web_url': {
                        return this.urlButton(button.title, button.url);
                    }
                    default: {
                        return this.postbackButton(button.title, button.payload);
                    }
                }
            })
        });
    }
    postbackButton(title, payload) {
        return {
            "type": "postBack",
            "title": title,
            "value": `${this._uid}-${payload}`
        };
    }
    urlButton(title, url) {
        return {
            "type": "openUrl",
            "title": title,
            "value": url
        };
    }
    attachment(card) {
        const template = {
            "contentType": "application/vnd.microsoft.card.hero",
            "content": card
        };
        return template;
    }
    appendQuickReplyTemplate(template, quickReplies) {
        if (!quickReplies || !quickReplies.length) {
            return;
        }
        template.content.buttons = quickReplies.map(button => {
            return {
                "title": button.title,
                "type": "postBack",
                "value": `${this._uid}-${button.payload}`
            };
        });
    }
}
exports.BotFrameworkExtension = BotFrameworkExtension;
