"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*jshint esversion: 6 */
const channel_1 = require("./channel");
class FBExtension extends channel_1.Channel {
    text(text, quickReplies) {
        const template = {
            text: text,
        };
        this.appendQuickReplyTemplate(template, quickReplies);
        return template;
    }
    image(url, quickReplies, assetId) {
        return this.attachment('image', url, quickReplies, assetId);
    }
    audio(url, quickReplies, assetId) {
        return this.attachment('audio', url, quickReplies, assetId);
    }
    video(url, quickReplies, assetId) {
        return this.attachment('video', url, quickReplies, assetId);
    }
    file(url, quickReplies, assetId) {
        return this.attachment('file', url, quickReplies, assetId);
    }
    genericTemplate(items) {
        const template = {
            "attachment": {
                "type": "template",
                "payload": {
                    "template_type": "generic",
                    "elements": items
                }
            }
        };
        return template;
    }
    genericItem(title, subtitle, imageUrl, defaultActionUrl, buttons, assetId) {
        let defaultAction;
        if (defaultActionUrl) {
            defaultAction = {
                "type": "web_url",
                url: defaultActionUrl
            };
        }
        return {
            "title": title,
            "subtitle": subtitle,
            "image_url": imageUrl,
            "default_action": defaultAction,
            "buttons": buttons
        };
    }
    compactListTemplate(elements) {
        const template = {
            "attachment": {
                "type": "template",
                "payload": {
                    "template_type": "list",
                    "top_element_style": "compact",
                    "elements": elements
                }
            }
        };
        return template;
    }
    listTemplate(topElementStyle = "large", elements) {
        const template = {
            "attachment": {
                "type": "template",
                "payload": {
                    "template_type": "list",
                    "top_element_style": topElementStyle,
                    "elements": elements
                }
            }
        };
        return template;
    }
    quickReplyButtonTemplate(text, buttons) {
        const template = {
            text: text,
            quick_replies: []
        };
        for (const button of buttons) {
            template.quick_replies.push({
                "content_type": "text",
                "title": button.title,
                "payload": button.payload,
                "image_url": button.imageUrl
            });
        }
        return template;
    }
    quickReplyButton(title, payload) {
        return {
            "title": title,
            "payload": payload || title
        };
    }
    buttonTemplate(text, buttons) {
        return {
            "attachment": {
                "type": "template",
                "payload": {
                    "template_type": "button",
                    "text": text,
                    "buttons": buttons
                }
            }
        };
    }
    postbackButton(title, blockName) {
        return {
            "type": "postback",
            "title": title,
            "payload": blockName
        };
    }
    replyButton(title, reply) {
        return {
            "type": "postback",
            "title": title,
            "payload": `reply-${reply}`
        };
    }
    callButton(title, phoneNumber) {
        return {
            "type": "phone_number",
            "title": title,
            "payload": phoneNumber
        };
    }
    shareButton(content) {
        return {
            "type": "element_share",
            "share_contents": content
        };
    }
    urlButton(title, url, attributes) {
        return Object.assign({ "type": "web_url", "title": title, "url": url }, attributes);
    }
    typing(duration) {
        return {
            "type": "typing",
            "duration": duration
        };
    }
    attachment(type, url, quickReplies, assetId) {
        const template = {
            "attachment": {
                "type": type,
                "payload": {}
            }
        };
        if (assetId) {
            template.attachment.payload.attachment_id = assetId;
        }
        else if (url) {
            template.attachment.payload.url = url.match(/{{[^\s]+}}/) ? url : encodeURI(url);
        }
        this.appendQuickReplyTemplate(template, quickReplies);
        return template;
    }
    appendQuickReplyTemplate(template, quickReplies) {
        if (!quickReplies || !quickReplies.length) {
            return;
        }
        template.quick_replies = quickReplies.map(button => {
            return {
                "content_type": "text",
                "title": button.title,
                "payload": button.payload,
                "image_url": button.imageUrl
            };
        });
    }
}
exports.FBExtension = FBExtension;
