"use strict";
/*jshint esversion: 6 */
Object.defineProperty(exports, "__esModule", { value: true });
const channel_1 = require("./channel");
class TGExtension extends channel_1.Channel {
    text(text, quickReplies) {
        const template = {
            type: "message",
            payload: {
                text: text,
                parse_mode: "Markdown"
            }
        };
        this.appInlineKeyboard(template, quickReplies);
        return template;
    }
    image(url, quickReplies, assetId) {
        const template = {
            type: "photo",
            payload: {
                photo: url
            }
        };
        this.appInlineKeyboard(template, quickReplies);
        return template;
    }
    genericTemplate(templates) {
        return templates.reduce((a, b) => {
            return a.concat(b);
        });
    }
    genericItem(title, subtitle, imageUrl, actionUrl, buttons, assetId) {
        let body = [];
        body.push(this.text(title));
        if (subtitle)
            body.push(this.text(subtitle));
        if (actionUrl)
            body.push(this.url(actionUrl));
        if (imageUrl)
            body.push(this.image(imageUrl, buttons));
        return body;
    }
    quickReplyButtonTemplate(text, buttons) {
        var template = {
            type: "message",
            payload: {
                text: text,
                reply_markup: {
                    keyboard: [],
                    resize_keyboard: true
                }
            }
        };
        // verticle.
        for (var button of buttons)
            template.payload.reply_markup.keyboard.push([button]);
        return template;
    }
    // facebook compatibilty
    quickReplyButton(title, payload, options) {
        return {
            text: title,
            request_contact: (payload && payload.request_contact) ||
                (options && options.request_contact),
            request_location: (payload && payload.request_location) ||
                (options && options.request_location)
        };
    }
    postbackButton(title, payload) {
        return {
            text: title
        };
    }
    buttonTemplate(text, quickReplies) {
        const template = {
            type: "message",
            payload: {
                text: text,
                parse_mode: "Markdown"
            }
        };
        this.appendKeyboard(template, quickReplies);
        return template;
    }
    url(url) {
        return this.text(`[${url}](${url})`);
    }
    typing() {
        return this.text("...");
    }
    replyButton(title, payload) {
        return this.postbackButton(title, `reply-${payload}`);
    }
    appendKeyboard(template, quickReplies) {
        if (quickReplies) {
            template.payload.reply_markup = {
                keyboard: [],
                resize_keyboard: true
            };
            for (var button of quickReplies) {
                template.payload.reply_markup.keyboard.push([
                    {
                        text: `/${button.title}`,
                        request_contact: button.request_contact,
                        request_location: button.request_location
                    }
                ]);
            }
        }
    }
    appInlineKeyboard(template, quickReplies) {
        if (quickReplies) {
            template.payload.reply_markup = {
                inline_keyboard: []
            };
            for (var button of quickReplies) {
                template.payload.reply_markup.inline_keyboard.push([
                    {
                        text: button.title,
                        callback_data: button.payload,
                        url: button.url
                    }
                ]);
            }
        }
    }
}
exports.TGExtension = TGExtension;
