"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*jshint esversion: 6 */
const channel_1 = require("./channel");
const index_1 = require("../index");
const MaxColumnSize = 6;
const MaxRowSize = 6;
class VBExtension extends channel_1.Channel {
    constructor(_uid) {
        super();
        this._uid = _uid;
    }
    text(text, quickReplies) {
        const template = {
            type: "text",
            text: text
        };
        this.appendQuickReplyTemplate(template, quickReplies);
        return template;
    }
    image(url, quickReplies, assetId) {
        const template = {
            type: "picture",
            text: "",
            media: url
        };
        this.appendQuickReplyTemplate(template, quickReplies);
        return template;
    }
    video(url, quickReplies, assetId, thumbnail, size) {
        const template = {
            type: "video",
            size: size || 26214400,
            media: url,
            thumbnail: thumbnail
        };
        this.appendQuickReplyTemplate(template, quickReplies);
        return template;
    }
    buttonTemplate(text, buttons) {
        let template = {
            type: "text",
            text: text,
            keyboard: {
                Type: "keyboard",
                // "DefaultHeight":true,
                Buttons: []
            }
        };
        for (var button of buttons) {
            var colSize = 0;
            buttons.map(b => {
                if (b.row === button.row && colSize < MaxColumnSize) {
                    colSize++;
                }
            });
            // naked object
            if (!(button instanceof index_1.Viber.Button)) {
                const style = new index_1.Viber.ButtonStyle();
                const properties = Object.getOwnPropertyNames(button);
                for (var property of properties) {
                    if (button.hasOwnProperty(property)) {
                        style[property] = button[property];
                    }
                }
                const temp = new index_1.Viber.Button(style);
                temp.payload = button.payload;
                temp.type = button.type;
                temp.text = button.title || button.text;
                temp.url = button.url;
                if (!(button.title || button.text)) {
                    temp.silent = true;
                }
                button = temp;
            }
            const gridButton = new index_1.Viber.GridButton(button);
            gridButton.rows = 1;
            gridButton.columns = MaxColumnSize / colSize;
            template.keyboard.Buttons.push(gridButton.toJSONObject());
        }
        return template;
    }
    genericTemplate(items) {
        const template = {
            type: "rich_media",
            min_api_version: 2
        };
        template.rich_media = {
            Type: "rich_media",
            ButtonsGroupColumns: 6,
            ButtonsGroupRows: 6,
            BgColor: "#FFFFFF",
            Buttons: items.reduce((a, b) => {
                return a.concat(b);
            }, [])
        };
        return template;
    }
    richMediaTemplate(items, style) {
        const template = {
            type: "rich_media",
            min_api_version: 2
        };
        template.rich_media = {
            Type: "rich_media",
            ButtonsGroupColumns: 6,
            ButtonsGroupRows: 6,
            BgColor: (style && style.bgColor) || "#FFFFFF",
            Buttons: items.reduce((a, b) => {
                return a.concat(b);
            }, [])
        };
        return template;
    }
    buttonGroup(buttons) {
        var gridButtons = [];
        for (var button of buttons) {
            // naked object
            if (!(button instanceof index_1.Viber.Button)) {
                const style = new index_1.Viber.ButtonStyle();
                const properties = Object.getOwnPropertyNames(button);
                for (var property of properties) {
                    if (button.hasOwnProperty(property)) {
                        style[property] = button[property];
                    }
                }
                const temp = new index_1.Viber.Button(style);
                temp.payload = button.payload;
                temp.type = button.type;
                temp.text = button.title || button.text;
                temp.url = button.url;
                temp.silent = button.slient;
                button = temp;
            }
            gridButtons.push(button.toJSONObject());
        }
        return gridButtons;
    }
    genericItem(title, subtitle, imageUrl, actionUrl, buttons) {
        const Title = {
            Columns: 6,
            Rows: 6,
            ActionType: "none",
            Text: title,
            TextHAlign: "left",
            TextSize: "large"
        };
        const rows = [Title];
        if (imageUrl) {
            rows[0].Rows = 1;
            rows.unshift({
                Columns: 6,
                Rows: 5,
                ActionType: "none",
                Image: imageUrl
            });
        }
        if (subtitle) {
            rows[0].Rows = parseInt(rows[0].Rows) - 1;
            rows.push({
                Columns: 6,
                Rows: 1,
                ActionType: "none",
                Text: subtitle,
                TextHAlign: "left"
            });
        }
        if (buttons) {
            for (var button of buttons) {
                rows[0].Rows = parseInt(rows[0].Rows) - 1;
                if (!(button instanceof index_1.Viber.Button)) {
                    // naked object
                    const style = new index_1.Viber.ButtonStyle();
                    const properties = Object.getOwnPropertyNames(button);
                    for (var property of properties) {
                        if (button.hasOwnProperty(property)) {
                            style[property] = button[property];
                        }
                    }
                    const vButton = new index_1.Viber.Button(style);
                    vButton.payload = button.payload;
                    vButton.type = button.type;
                    vButton.text = button.title || button.text;
                    vButton.url = button.url;
                    button = vButton;
                }
                const vb = new index_1.Viber.GridButton(button);
                vb.rows = 1;
                vb.columns = 6;
                rows.push(vb.toJSONObject());
            }
        }
        return rows;
    }
    postbackButton(title, payload, style) {
        // viber default
        if (!style) {
            style = new index_1.Viber.ButtonStyle();
        }
        const button = new index_1.Viber.Button(style);
        button.text = title;
        button.type = "postback";
        button.payload = `${this._uid}-${payload}`;
        return button;
    }
    textButton(text, style) {
        // viber default
        if (!style) {
            style = new index_1.Viber.ButtonStyle();
        }
        const button = new index_1.Viber.Button(style);
        button.text = text;
        button.type = "none";
        return button;
    }
    imageButton(style) {
        // viber default
        if (!style) {
            style = new index_1.Viber.ButtonStyle();
        }
        const button = new index_1.Viber.Button(style);
        button.type = "none";
        return button;
    }
    replyButton(title, payload, style) {
        // viber default
        if (!style) {
            style = new index_1.Viber.ButtonStyle();
        }
        const button = new index_1.Viber.Button(style);
        button.text = title;
        button.type = "postback";
        button.payload = `reply-${payload}`;
        return button;
    }
    sticker(ids) {
        if (Array.isArray(ids)) {
            if (ids.length > 1) {
                const stickerId = Math.floor(Math.random() * ids.length);
                return {
                    type: "sticker",
                    sticker_id: ids[stickerId]
                };
            }
            else {
                return {
                    type: "sticker",
                    sticker_id: ids[0]
                };
            }
        }
        else {
            return {
                type: "sticker",
                sticker_id: ids
            };
        }
    }
    typing(duration) {
        return {
            type: "typing",
            duration: duration
        };
    }
    urlButton(title, url, style) {
        // viber default
        if (!style) {
            style = new index_1.Viber.ButtonStyle();
        }
        const button = new index_1.Viber.Button(style);
        button.text = title;
        button.type = "web_url";
        button.url = url;
        button.silent = true;
        return button;
    }
    // facebook compatibility
    quickReplyButtonTemplate(text, buttons) {
        return this.buttonTemplate(text, buttons);
    }
    // facebook compatibility.
    quickReplyButton(text, payload, style) {
        return this.postbackButton(text, payload, style);
    }
    appendQuickReplyTemplate(template, quickReplies) {
        if (quickReplies) {
            template.keyboard = {
                Type: "keyboard",
                // "DefaultHeight": true,
                Buttons: []
            };
            // viber default
            const style = new index_1.Viber.ButtonStyle();
            for (var button of quickReplies) {
                const defaultButton = new index_1.Viber.Button(style);
                defaultButton.text = button.title;
                defaultButton.payload = `${process.env.UID}-${button.payload}`;
                template.keyboard.Buttons.push(defaultButton.toString());
            }
        }
    }
}
exports.VBExtension = VBExtension;
