"use strict";
/*jshint esversion: 6 */
Object.defineProperty(exports, "__esModule", { value: true });
const channel_1 = require("./channel");
class WebExtension extends channel_1.Channel {
    // Text response
    text(text, quickReplies) {
        const template = {
            "type": "text",
            text: text,
        };
        this.processQuickReplies(template, quickReplies);
        return template;
    }
    image(url, quickReplies) {
        const template = {
            "type": "image",
            "image": url
        };
        this.processQuickReplies(template, quickReplies);
        return template;
    }
    audio(url, quickReplies) {
        const template = {
            "type": "audio",
            "audio": url
        };
        this.processQuickReplies(template, quickReplies);
        return template;
    }
    video(url, quickReplies) {
        const template = {
            "type": "video",
            "video": url
        };
        this.processQuickReplies(template, quickReplies);
        return template;
    }
    processQuickReplies(template, quickReplies) {
        if (Array.isArray(quickReplies) && quickReplies.length) {
            template.quickReplies = (quickReplies || []).map((button) => {
                if (button.type === 'postback') {
                    return this.postbackButton(button.title, button.payload, button.attributes);
                }
                if (button.type === 'reply') {
                    return this.replyButton(button.title, button.reply, button.attributes);
                }
            });
        }
    }
    replyButton(title, reply, attributes) {
        return this.postbackButton(title, `reply-${reply}`, attributes);
    }
    postbackButton(title, blockName, attributes) {
        return Object.assign({ "type": "postback", "title": title, "postback": blockName }, attributes);
    }
    urlButton(title, url, attributes) {
        return Object.assign({ "type": "url", "title": title, "url": url }, attributes);
    }
    buttonTemplate(text, buttons) {
        return {
            "type": "button-template",
            "template": {
                "text": text,
                "buttons": (buttons || []).map(button => {
                    if (button.type === 'postback') {
                        return this.postbackButton(button.title, button.payload, button.attributes);
                    }
                    if (button.type === 'reply') {
                        return this.replyButton(button.title, button.reply, button.attributes);
                    }
                    return this.urlButton(button.title, button.url, button.attributes);
                })
            }
        };
    }
    genericTemplate(items) {
        const template = {
            "type": "generic-template",
            "template": {
                "elements": items
            }
        };
        return template;
    }
    typing(duration) {
        return {
            "type": "typing",
            "duration": duration
        };
    }
    genericItem(title, subtitle, imageUrl, defaultActionUrl, buttons, assetId) {
        let defaultAction;
        if (defaultActionUrl) {
            defaultAction = {
                "type": "url",
                url: defaultActionUrl
            };
        }
        return {
            "title": title,
            "subtitle": subtitle,
            "imageUrl": imageUrl,
            "defaultAction": defaultAction,
            "buttons": (buttons || []).map(button => {
                if (button.type === 'postback') {
                    return this.postbackButton(button.title, button.payload, button.attributes);
                }
                if (button.type === 'reply') {
                    return this.replyButton(button.title, button.reply, button.attributes);
                }
                return this.urlButton(button.title, button.url, button.attributes);
            })
        };
    }
    sticker(ids) {
        if (Array.isArray(ids)) {
            if (ids.length > 1) {
                const stickerId = Math.floor(Math.random() * ids.length);
                return this.text(`Sticker placeholder: ${ids[stickerId]}`);
            }
            else {
                return this.text(`Sticker placeholder: ${ids[0]}`);
            }
        }
        return this.text(`Sticker placeholder: ${ids}`);
    }
    richMediaTemplate(items, attributes) {
        return this.text("Viber rich media is not supported in the simulator. Test it on Viber.");
    }
    buttonGroup(buttons) {
        return buttons;
    }
}
exports.WebExtension = WebExtension;
