"use strict";
// Copyright 2018 The Recime Inc. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
const bot_framework_1 = require("./ext/bot-framework");
const facebook_1 = require("./ext/facebook");
const kik_1 = require("./ext/kik");
const wechat_1 = require("./ext/wechat");
const telegram_1 = require("./ext/telegram");
const twilio_1 = require("./ext/twilio");
const sendbird_1 = require("./ext/sendbird");
const slack_1 = require("./ext/slack");
const viber_1 = require("./ext/viber");
const web_1 = require("./ext/web");
var Viber;
(function (Viber) {
    let TextSize;
    (function (TextSize) {
        TextSize[TextSize["regular"] = 0] = "regular";
        TextSize[TextSize["medium"] = 1] = "medium";
        TextSize[TextSize["large"] = 2] = "large";
    })(TextSize = Viber.TextSize || (Viber.TextSize = {}));
    let ActionType;
    (function (ActionType) {
        ActionType[ActionType["reply"] = 0] = "reply";
        ActionType[ActionType["open-url"] = 1] = "open-url";
    })(ActionType = Viber.ActionType || (Viber.ActionType = {}));
    let TextVAlign;
    (function (TextVAlign) {
        TextVAlign[TextVAlign["top"] = 0] = "top";
        TextVAlign[TextVAlign["middle"] = 1] = "middle";
        TextVAlign[TextVAlign["bottom"] = 2] = "bottom";
    })(TextVAlign = Viber.TextVAlign || (Viber.TextVAlign = {}));
    let TextHAlign;
    (function (TextHAlign) {
        TextHAlign[TextHAlign["left"] = 0] = "left";
        TextHAlign[TextHAlign["center"] = 1] = "center";
        TextHAlign[TextHAlign["right"] = 2] = "right";
    })(TextHAlign = Viber.TextHAlign || (Viber.TextHAlign = {}));
    class FontImpl {
        constructor(text) {
            this.text = text;
        }
        set color(color) {
            this._color = color;
        }
        toJSONObject() {
            if (this._color && this.text) {
                return `<font color=\"${this._color}\">${this.text}</font>`;
            }
            return this.text || "";
        }
    }
    Viber.FontImpl = FontImpl;
    class FontBold extends FontImpl {
        constructor(text) {
            super(text);
        }
        toJSONObject() {
            return `<b>${super.toJSONObject()}</b>`;
        }
    }
    class ButtonStyle {
        toJSONObject() {
            var bgColor = "#FFFFFF";
            if (this.bgColor && this.bgColor.match(/^[#]([A-Fa-f0-9]{6})$/ig)) {
                bgColor = this.bgColor;
            }
            var template = {
                "TextSize": this.textSize || "regular",
                "BgColor": bgColor,
                "TextHAlign": this.align || "center",
                "TextVAlign": this.vAlign || 'middle',
                "TextOpacity": this.textOpacity || 100,
            };
            if (this.rows && this.columns) {
                template["Rows"] = this.rows;
                template["Columns"] = this.columns;
            }
            if (this.imageUrl) {
                template["Image"] = this.imageUrl;
            }
            return template;
        }
    }
    Viber.ButtonStyle = ButtonStyle;
    class Button {
        constructor(style) {
            this.style = new ButtonStyle();
            Object.assign(this.style, style);
        }
        ;
        get row() {
            return this.style.row;
        }
        get column() {
            return this.style.column;
        }
        get rows() {
            return this.style.rows;
        }
        get columns() {
            return this.style.columns;
        }
        set type(type) {
            this._type = type;
        }
        set text(text) {
            this._text = text;
        }
        set payload(payload) {
            this._payload = payload;
        }
        set silent(slient) {
            this._slient = slient;
        }
        set url(url) {
            this._url = url;
        }
        toJSONObject() {
            const style = this.style.toJSONObject();
            let fontColor = "#000000";
            if (this.style.fgColor && this.style.fgColor.match(/^[#]([A-Fa-f0-9]{6})$/ig)) {
                fontColor = this.style.fgColor;
            }
            let font = new FontImpl(this._text);
            if (this.style.bold) {
                font = new FontBold(this._text);
            }
            font.color = fontColor;
            const text = font.toJSONObject();
            const isWebUrl = (type) => type === 'web_url';
            if (this._type === 'none') {
                return Object.assign({
                    "ActionType": "none",
                    "Text": text
                }, style);
            }
            const template = Object.assign({
                "ActionType": isWebUrl(this._type) ? "open-url" : "reply",
                "ActionBody": isWebUrl(this._type) ? this._url : `${process.env.UID}-${this._payload}`,
                "Text": text,
                "Silent": this._slient || false
            }, style);
            if (isWebUrl(this._type)) {
                return Object.assign({
                    "OpenURLType": "internal",
                    "InternalBrowser": {
                        "Mode": "fullscreen"
                    }
                }, template);
            }
            return template;
        }
    }
    Viber.Button = Button;
    class GridButton {
        constructor(button) {
            this._button = button;
        }
        toJSONObject() {
            return Object.assign({
                "Rows": this.rows,
                "Columns": this.columns
            }, this._button.toJSONObject());
        }
    }
    Viber.GridButton = GridButton;
})(Viber = exports.Viber || (exports.Viber = {}));
// Extension root.
class Ext {
    static get "bot-framework"() {
        return new bot_framework_1.BotFrameworkExtension(process.env.UID);
    }
    static get facebook() {
        return new facebook_1.FBExtension();
    }
    static get viber() {
        return new viber_1.VBExtension(process.env.UID);
    }
    static get kik() {
        return new kik_1.KikExtension();
    }
    static get telegram() {
        return new telegram_1.TGExtension();
    }
    static get twilio() {
        return new twilio_1.TwilioExtension();
    }
    static get slack() {
        return new slack_1.SlackExtension();
    }
    static get sendbird() {
        return new sendbird_1.SendBirdExtension();
    }
    static get web() {
        return new web_1.WebExtension();
    }
    static get wechat() {
        return new wechat_1.WCExtension();
    }
    // common operations, available in all platform.
    static get default() {
        if (process.env.PLATFORM) {
            return this[process.env.PLATFORM];
        }
        throw new Error('Platform not supported!');
    }
}
exports.default = Ext;
